@@ -1,5 +1,5 @@
 select distinct name
-from   all_source
+from   dba_source
 where  UPPER(owner) = UPPER( ? )
 and    type = 'PROCEDURE'
 order by name
